@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -51,13 +51,13 @@ public:
         jassert (instance != nullptr);
         instance = nullptr;
 
-        if (handle != nullptr)
-            snd_seq_close (handle);
-
         jassert (activeCallbacks.get() == 0);
 
         if (inputThread)
             inputThread->stopThread (3000);
+
+        if (handle != nullptr)
+            snd_seq_close (handle);
     }
 
     static String getAlsaMidiName()
@@ -123,10 +123,10 @@ public:
 
         void enableCallback (bool enable)
         {
-            if (callbackEnabled != enable)
-            {
-                callbackEnabled = enable;
+            const auto oldValue = callbackEnabled.exchange (enable);
 
+            if (oldValue != enable)
+            {
                 if (enable)
                     client.registerCallback();
                 else
@@ -203,14 +203,20 @@ public:
 
         void handleIncomingMidiMessage (const MidiMessage& message) const
         {
-            callback->handleIncomingMidiMessage (midiInput, message);
+            if (callbackEnabled)
+                callback->handleIncomingMidiMessage (midiInput, message);
         }
 
         void handlePartialSysexMessage (const uint8* messageData, int numBytesSoFar, double timeStamp)
         {
-            callback->handlePartialSysexMessage (midiInput, messageData, numBytesSoFar, timeStamp);
+            if (callbackEnabled)
+                callback->handlePartialSysexMessage (midiInput, messageData, numBytesSoFar, timeStamp);
         }
 
+        int getPortId() const               { return portId; }
+        const String& getPortName() const   { return portName; }
+
+    private:
         AlsaClient& client;
 
         MidiInputCallback* callback = nullptr;
@@ -220,7 +226,8 @@ public:
         String portName;
 
         int maxEventSize = 4096, portId = -1;
-        bool callbackEnabled = false, isInput = false;
+        std::atomic<bool> callbackEnabled { false };
+        bool isInput = false;
     };
 
     static Ptr getInstance()
@@ -250,15 +257,18 @@ public:
 
     void handleIncomingMidiMessage (snd_seq_event* event, const MidiMessage& message)
     {
-        if (event->dest.port < ports.size() && ports[event->dest.port]->callbackEnabled)
-            ports[event->dest.port]->handleIncomingMidiMessage (message);
+        const ScopedLock sl (callbackLock);
+
+        if (auto* port = ports[event->dest.port])
+            port->handleIncomingMidiMessage (message);
     }
 
     void handlePartialSysexMessage (snd_seq_event* event, const uint8* messageData, int numBytesSoFar, double timeStamp)
     {
-        if (event->dest.port < ports.size()
-            && ports[event->dest.port]->callbackEnabled)
-            ports[event->dest.port]->handlePartialSysexMessage (messageData, numBytesSoFar, timeStamp);
+        const ScopedLock sl (callbackLock);
+
+        if (auto* port = ports[event->dest.port])
+            port->handlePartialSysexMessage (messageData, numBytesSoFar, timeStamp);
     }
 
     snd_seq_t* get() const noexcept     { return handle; }
@@ -266,16 +276,20 @@ public:
 
     Port* createPort (const String& name, bool forInput, bool enableSubscription)
     {
+        const ScopedLock sl (callbackLock);
+
         auto port = new Port (*this, forInput);
         port->createPort (name, enableSubscription);
-        ports.set (port->portId, port);
+        ports.set (port->getPortId(), port);
         incReferenceCount();
         return port;
     }
 
     void deletePort (Port* port)
     {
-        ports.set (port->portId, nullptr);
+        const ScopedLock sl (callbackLock);
+
+        ports.set (port->getPortId(), nullptr);
         decReferenceCount();
     }
 
@@ -492,7 +506,7 @@ std::unique_ptr<MidiInput> MidiInput::openDevice (const String& deviceIdentifier
 
     jassert (port->isValid());
 
-    std::unique_ptr<MidiInput> midiInput (new MidiInput (port->portName, deviceIdentifier));
+    std::unique_ptr<MidiInput> midiInput (new MidiInput (port->getPortName(), deviceIdentifier));
 
     port->setupInput (midiInput.get(), callback);
     midiInput->internal = std::make_unique<Pimpl> (port);
@@ -508,7 +522,7 @@ std::unique_ptr<MidiInput> MidiInput::createNewDevice (const String& deviceName,
     if (port == nullptr || ! port->isValid())
         return {};
 
-    std::unique_ptr<MidiInput> midiInput (new MidiInput (deviceName, getFormattedPortIdentifier (client->getId(), port->portId)));
+    std::unique_ptr<MidiInput> midiInput (new MidiInput (deviceName, getFormattedPortIdentifier (client->getId(), port->getPortId())));
 
     port->setupInput (midiInput.get(), callback);
     midiInput->internal = std::make_unique<Pimpl> (port);
@@ -589,7 +603,7 @@ std::unique_ptr<MidiOutput> MidiOutput::openDevice (const String& deviceIdentifi
     if (port == nullptr || ! port->isValid())
         return {};
 
-    std::unique_ptr<MidiOutput> midiOutput (new MidiOutput (port->portName, deviceIdentifier));
+    std::unique_ptr<MidiOutput> midiOutput (new MidiOutput (port->getPortName(), deviceIdentifier));
 
     port->setupOutput();
     midiOutput->internal = std::make_unique<Pimpl> (port);
@@ -605,7 +619,7 @@ std::unique_ptr<MidiOutput> MidiOutput::createNewDevice (const String& deviceNam
     if (port == nullptr || ! port->isValid())
         return {};
 
-    std::unique_ptr<MidiOutput> midiOutput (new MidiOutput (deviceName, getFormattedPortIdentifier (client->getId(), port->portId)));
+    std::unique_ptr<MidiOutput> midiOutput (new MidiOutput (deviceName, getFormattedPortIdentifier (client->getId(), port->getPortId())));
 
     port->setupOutput();
     midiOutput->internal = std::make_unique<Pimpl> (port);
