@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,6 +23,8 @@
 namespace juce
 {
 
+#if ! JUCE_WASM
+
 JUCE_BEGIN_IGNORE_WARNINGS_MSVC (4127 4389 4018)
 
 #ifndef AI_NUMERICSERV  // (missing in older Mac SDKs)
@@ -56,11 +58,9 @@ namespace SocketHelpers
 
         if (! socketsStarted)
         {
-            socketsStarted = true;
-
             WSADATA wsaData;
             const WORD wVersionRequested = MAKEWORD (1, 1);
-            WSAStartup (wVersionRequested, &wsaData);
+            socketsStarted = WSAStartup (wVersionRequested, &wsaData) == 0;
         }
        #endif
     }
@@ -130,7 +130,7 @@ namespace SocketHelpers
                 // a chance to process before close is called. On Mac OS X shutdown
                 // does not unblock a select call, so using a lock here will dead-lock
                 // both threads.
-               #if JUCE_LINUX || JUCE_ANDROID
+               #if JUCE_LINUX || JUCE_BSD || JUCE_ANDROID
                 CriticalSection::ScopedLockType lock (readLock);
                 ::close (h);
                #else
@@ -778,7 +778,7 @@ bool DatagramSocket::setEnablePortReuse (bool enabled)
    #else
     if (handle >= 0)
         return SocketHelpers::setOption ((SocketHandle) handle.load(),
-                                        #if JUCE_WINDOWS || JUCE_LINUX
+                                        #if JUCE_WINDOWS || JUCE_LINUX || JUCE_BSD
                                          SO_REUSEADDR,  // port re-use is implied by addr re-use on these platforms
                                         #else
                                          SO_REUSEPORT,
@@ -857,6 +857,7 @@ struct SocketTests : public UnitTest
 
 static SocketTests socketTests;
 
+#endif
 #endif
 
 } // namespace juce
